@@ -15,6 +15,74 @@ import time
 import sys
 import keyboard
 import matplotlib.pyplot as plt
+import re
+
+operatorsDict = {"*": 2, "/": 2, "+": 1, "%": 2, '^': 4, "-": 1}
+operators = operatorsDict.keys()
+
+def tokenize(inputExpression):
+    return re.split("([()" + ''.join(operators) + "])", inputExpression.replace(" ", ""))
+
+def shunting_yard(inputExpression):
+    queue = []
+    stack = []
+
+    tokens = tokenize(inputExpression)
+    tokens = [x for x in tokens if x]
+    for token in tokens:
+        if(token not in operators and token != "(" and token != ")"):
+            queue.append(token)
+
+        if(token == "("):
+            stack.append(token)
+       
+        if(token == ")"):
+            while(len(stack) != 0 and stack[-1] != "("):
+                queue.append(stack.pop())
+     
+            stack.pop()
+       
+        if(token in operators):
+            while(len(stack) != 0 and stack[-1] in operators and operatorsDict[stack[-1]] >= operatorsDict[token]):
+                queue.append(stack.pop())
+
+            stack.append(token)
+
+    while(len(stack) != 0):
+        queue.append(stack.pop())
+
+    return queue
+
+def evaluatePostfix(inputList):
+    stack = []
+
+    for token in inputList:
+        if(token not in operators):
+            stack.append(token)
+        else:
+            operand2 = float(stack.pop())
+            operand1 = float(stack.pop())
+
+            if(token == "+"):
+                stack.append(operand1 + operand2)
+
+            if(token == "-"):
+                stack.append(operand1 - operand2)
+
+            if(token == "*"):
+                stack.append(operand1 * operand2)
+
+            if(token == "/"):
+                stack.append(operand1 / operand2)
+
+            if(token == '^'):
+                stack.append(operand1 ** operand2)
+               
+    return stack[0]
+
+def evaluateExpression(inputExpression):
+    return evaluatePostfix(shunting_yard(inputExpression))
+
 
 def teach_me_trig():
     print("welcome to the trig basics course!")
@@ -278,7 +346,7 @@ def bt(x):
     data.set(val)
 def do_it():
     global val
-    y = eval(val)
+    y = evaluateExpression(val)
     data.set(y)
     ans = val
     val=""
@@ -686,7 +754,7 @@ def nthFinder(a, b):
     a = str(a)
     b = str(b)
     x = a.replace("n", b)
-    y = eval(x)
+    y = evaluateExpression(x)
     return y
 def nthRange(a, b, c):
     ls = []
@@ -696,7 +764,7 @@ def nthRange(a, b, c):
     if int(b) > int(c):
         while int(c) < int(b)+1:
             x = a.replace("n", c)
-            y = eval(x)
+            y = evaluateExpression(x)
             ls.append(y)
             c = int(c)
             c=int(c)+1
@@ -704,7 +772,7 @@ def nthRange(a, b, c):
     elif int(c) > int(b):
         while int(b) < int(c)+1:
             x = a.replace("n", b)
-            y = eval(x)
+            y = evaluateExpression(x)
             ls.append(y)
             b = int(b)
             b=int(b)+1
@@ -718,7 +786,7 @@ def nthTable(a, b, c):
     if int(b) > int(c):
         while int(c) < int(b)+1:
             x = a.replace("n", c)
-            y = eval(x)
+            y = evaluateExpression(x)
             q = [c, y]
             ls.append(q)
             ls.append(q)
@@ -728,7 +796,7 @@ def nthTable(a, b, c):
     elif int(c) > int(b):
         while int(b) < int(c)+1:
             x = a.replace("n", b)
-            y = eval(x)
+            y = evaluateExpression(x)
             q = [b, y]
             ls.append(q)
             b = int(b)
@@ -885,20 +953,12 @@ def constTable():
     thing = tabulate(conlist, headers = headers)
     print(thing)
 
-def graph(graph_func, lrange=-10, urange=10, graph_points=100, graph_title=None):
-    x = np.linspace(lrange, urange, graph_points)
-    y = eval(graph_func)
-    plt.title(graph_title)
-    plt.plot(x,y) 
-    plt.show()
-def exp_graph(*argv):
-    z = 0
-    argv = list(argv)
-    while z < len(argv):
+def graph(*args, lrange=-10, urange=10, graph_points=100, graph_title=None):
+    i = 0
+    while i < len(args):
         x = np.linspace(lrange, urange, graph_points)
-        y = eval(str(argv[z]))
+        y = evaluateExpression(args[i])
         plt.title(graph_title)
         plt.plot(x,y)
-        z += 1
+        i+=1
     plt.show()
-
